#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;

#endregion

namespace Web.YUI
{
    [ParseChildren(true), PersistChildren(false)]
    public class Button : StateManagedCompositeControl, IYahooControl
    {
        #region Fields

        private ButtonGroup group = null;
        private EventCollection events = null;

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public new EventCollection Events
        {
            get
            {
                if (this.events == null)
                {
                    this.events = new EventCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.events.TrackViewState();
                    }
                }
                return this.events;
            }
        }

        public ButtonGroup Group
        {
            get { return group; }
            set { group = value; }
        }

        public ButtonType Type
        {
            get
            {
                object o = ViewState["ButtonType"];
                return o == null ? ButtonType.Button : (ButtonType)o;
            }
            set { ViewState["ButtonType"] = value; }
        }

        public string Text
        {
            get
            {
                object o = ViewState["Text"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Text"] = value; }
        }

        public string Value
        {
            get
            {
                object o = ViewState["Value"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Value"] = value; }
        }

        public string Title
        {
            get
            {
                object o = ViewState["Title"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Title"] = value; }
        }

        public string Name
        {
            get
            {
                object o = ViewState["Name"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Name"] = value; }
        }

        public string NavigateUrl
        {
            get
            {
                object o = ViewState["NavigateUrl"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["NavigateUrl"] = value; }
        }

        public string Target
        {
            get
            {
                object o = ViewState["Target"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Target"] = value; }
        }

        public bool Checked
        {
            get
            {
                object o = ViewState["Checked"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["Checked"] = value; }
        }

        public string MenuID
        {
            get
            {
                object o = ViewState["MenuID"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["MenuID"] = value; }
        }

        public string ContainerID
        {
            get
            {
                object o = ViewState["ContainerID"];
                return o == null ? String.Format("{0}_container", ID) : (string)o;
            }
            set { ViewState["ContainerID"] = value; }
        }

        public string ImageUrl
        {
            get
            {
                object o = ViewState["ImageUrl"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["ImageUrl"] = value; }
        }

        #endregion

        #region Overriden Methods

        protected override void Render(HtmlTextWriter writer)
        {
            CssClass = "yuibutton";
            writer.WriteBeginTag("span");
            HtmlHelper.WriteStyle(ControlStyle, writer);
            writer.WriteAttribute("id", ContainerID);
            foreach (string key in Attributes.Keys)
            {
                writer.WriteAttribute(key, Attributes[key]);
            }
            writer.Write(HtmlTextWriter.TagRightChar);
            
            writer.WriteBeginTag("span");
            writer.WriteAttribute("class", "first-child");
            writer.Write(HtmlTextWriter.TagRightChar);

            if (Type == ButtonType.Link)
            {
                HyperLink link = new HyperLink();
                link.NavigateUrl = NavigateUrl;
                link.Text = Text;
                link.RenderControl(writer);
            }
            else
            {
                writer.WriteBeginTag("button");
                writer.WriteAttribute("type", "button");
                //writer.WriteAttribute("type", Type.ToString().ToLower());
                if (!String.IsNullOrEmpty(Name))
                {
                    writer.WriteAttribute("name", Name);
                }
                writer.Write(HtmlTextWriter.TagRightChar);
                if (String.IsNullOrEmpty(ImageUrl))
                {
                    writer.Write(Text);
                }
                else
                {
                    Image img = new Image();
                    img.ImageUrl = ImageUrl;
                    img.RenderControl(writer);
                }
                writer.WriteEndTag("button");
            }
            
            writer.WriteEndTag("span");
            
            writer.WriteEndTag("span");
            writer.WriteLine();
        }

        #endregion

        #region IYahooControl Members

        public string LoadScript
        {
            get
            {
                if (Group != null)
                    return String.Empty;
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = new DelimitedList<string>(", ");
                options.Add(String.Format("type: \"{0}\"", Type.ToString().ToLower()));
                if (Checked)
                    options.Add("checked: true");
                if (!String.IsNullOrEmpty(Name))
                    options.Add(String.Format("name: \"{0}\"", Name));
                if (!String.IsNullOrEmpty(Value))
                    options.Add(String.Format("value: \"{0}\"", Value));
                if (!String.IsNullOrEmpty(Text))
                    options.Add(String.Format("label: \"{0}\"", Text));
                if (TabIndex > 0)
                    options.Add(String.Format("tabindex: {0}", TabIndex));
                if (!String.IsNullOrEmpty(Title))
                    options.Add(String.Format("title: \"{0}\"", Title));
                if (!Enabled)
                    options.Add("disabled: true");
                if (!String.IsNullOrEmpty(NavigateUrl))
                    options.Add(String.Format("href: \"{0}\"", NavigateUrl));
                if (!String.IsNullOrEmpty(Target))
                    options.Add(String.Format("target: \"{0}\"", Target));
                if (!String.IsNullOrEmpty(MenuID))
                    options.Add(String.Format("menu: \"{0}\"", MenuID));
                s.AppendFormat("var {0} = new YAHOO.widget.Button(\"{1}\",", ID, ContainerID);
                s.Append(" { ");
                s.Append(options.ToString());
                s.Append("} );");
                if (Events.Count > 0)
                {
                    foreach (Event e in Events)
                    {
                        if (String.IsNullOrEmpty(e.Tag))
                        {
                            s.AppendFormat("\n\t\t{0}.addListener(\"{1}\", {2});", ID, e.Name, e.Handler);
                        }
                        else
                        {
                            s.AppendFormat("\n\t\t{0}.addHandler(\"{1}\", {2}, \"{3}\");", ID, e.Name, e.Handler, e.Tag);
                        }
                    }
                }
                return s.ToString();
            }
        }

        public YahooScript[] ScriptDependencies
        {
            get
            {
                List<YahooScript> scripts = new List<YahooScript>();
                scripts.Add(YahooScript.Yahoo_Dom_Event);
                scripts.Add(YahooScript.Element);
                if (Type == ButtonType.MenuButton || Type == ButtonType.SplitButton)
                {
                    scripts.Add(YahooScript.Container);
                    scripts.Add(YahooScript.Menu);
                }
                scripts.Add(YahooScript.Button);
                return scripts.ToArray();
            }
        }

        public YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Fonts, YahooStyleSheet.Button }; }
        }

        #endregion
    }
}