#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.ComponentModel;

#endregion

namespace Web.YUI
{
    [ParseChildren(true), PersistChildren(false)]
    public class ButtonGroup : System.Web.UI.WebControls.CompositeControl, IYahooControl
    {
        #region Fields

        private ButtonCollection items = null;

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public ButtonCollection Items
        {
            get
            {
                if (this.items == null)
                {
                    this.items = new ButtonCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.items.TrackViewState();
                    }
                }
                return this.items;
            }
        }

        public ButtonType Type
        {
            get
            {
                object o = ViewState["ButtonType"];
                return o == null ? ButtonType.Button : (ButtonType)o;
            }
            set { ViewState["ButtonType"] = value; }
        }

        #endregion

        #region Methods

        protected override void CreateChildControls()
        {
            base.CreateChildControls();
            foreach (Button b in Items)
            {
                b.Type = Type;
                b.Name = String.Format("{0}_items", ID);
                b.Group = this;
                Controls.Add(b);
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            writer.WriteBeginTag("div");
            writer.WriteAttribute("id", ID);
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteLine();
            writer.Indent++;

            foreach (Button b in Items)
            {
                b.RenderControl(writer);
            }

            writer.Indent--;
            writer.WriteEndTag("div");
        }

        #endregion

        #region IYahooControl Members

        public string LoadScript
        {
            get
            {
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = new DelimitedList<string>(", ");
                if (!Enabled)
                    options.Add("disabled: true");
                if (options.Count > 0)
                {
                    s.AppendFormat("var {0} = new YAHOO.widget.Button(\"{0}\",", ID);
                    s.Append(" { ");
                    s.Append(options.ToString());
                    s.Append("} );");
                }
                else
                {
                    s.AppendFormat("var {0} = new YAHOO.widget.ButtonGroup(\"{0}\");", ID);
                }
                return s.ToString();
            }
        }

        public YahooScript[] ScriptDependencies
        {
            get
            {
                List<YahooScript> scripts = new List<YahooScript>();
                scripts.Add(YahooScript.Yahoo_Dom_Event);
                scripts.Add(YahooScript.Element);
                if (Type == ButtonType.MenuButton || Type == ButtonType.SplitButton)
                {
                    scripts.Add(YahooScript.Container);
                    scripts.Add(YahooScript.Menu);
                }
                return scripts.ToArray();
            }
        }

        public YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Fonts, YahooStyleSheet.Button }; }
        }

        #endregion
    }
}