#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;

#endregion

namespace Web.YUI
{
    public class Calendar : System.Web.UI.WebControls.CompositeControl, IYahooControl
    {
        #region Properties

        public string ContainerID
        {
            get
            {
                object o = ViewState["ContainerID"];
                return o == null ? String.Format("{0}_container", ID) : (string)o;
            }
            set { ViewState["ContainerID"] = value; }
        }

        private string HiddenFieldID
        {
            get { return String.Format("{0}_selectedDates", ID); }
        }

        /// <summary>
        /// Sets the calendar's selected dates. The built-in default date format is MM/DD/YYYY. Ranges are defined using MM/DD/YYYY-MM/DD/YYYY. Month/day combinations are defined using MM/DD. Any combination of these can be combined by delimiting the string with commas. For example: "12/24/2005,12/25/2005,1/18/2006-1/21/2006"
        /// </summary>
        public string SelectedDateString
        {
            get
            {
                DateTime[] dates = SelectedDates;
                if (dates != null)
                {
                    DelimitedList<string> l = new DelimitedList<string>();
                    foreach (DateTime d in dates)
                    {
                        l.Add(String.Format("{0:d}", d));
                    }
                    return l.ToString();
                }
                return String.Empty;
            }
            set { ViewState["SelectedDateString"] = value; }
        }

        public DateTime[] SelectedDates
        {
            get
            {
                string dateString = Page.Request.Form[HiddenFieldID];
                if(String.IsNullOrEmpty(dateString))
                {
                    object o = ViewState["SelectedDateString"];
                    dateString = (o == null ? String.Empty : (string)o);
                }

                if (!String.IsNullOrEmpty(dateString))
                {
                    DelimitedList<string> l = DelimitedList<string>.Parse(dateString);
                    List<DateTime> dates = new List<DateTime>();
                    foreach (string s in l)
                    {
                        DateTime d;
                        if (DateTime.TryParse(s, out d))
                        {
                            dates.Add(d);
                        }
                        else if (s.Contains("-"))
                        {
                            //it is a date range
                            string[] parts = s.Split('-');
                            DateTime start;
                            DateTime end;
                            if (DateTime.TryParse(parts[0], out start) &&
                                DateTime.TryParse(parts[1], out end))
                            {
                                System.Web.UI.WebControls.SelectedDatesCollection range = new System.Web.UI.WebControls.SelectedDatesCollection(new System.Collections.ArrayList());
                                range.SelectRange(start, end);
                                foreach (DateTime sd in range)
                                {
                                    dates.Add(sd);
                                }
                            }
                        }
                    }
                    return dates.ToArray();
                }
                return null;
            }
        }

        public DateTime? SelectedDate
        {
            get
            {
                DateTime[] dates = SelectedDates;
                if (dates == null || dates.Length < 1)
                    return null;
                return new DateTime?(dates[0]);
            }
        }

        /// <summary>
        /// Gets or sets the calendar's visible month and year
        /// </summary>
        public DateTime PageDate
        {
            get
            {
                object o = ViewState["PageDate"];
                return o == null ? DateTime.Today : (DateTime)o;
            }
            set { ViewState["PageDate"] = value; }
        }

        /// <summary>
        /// Gets or sets the Calendar's minimum selectable date
        /// </summary>
        public Nullable<DateTime> MinDate
        {
            get
            {
                object o = ViewState["MinDate"];
                return o == null ? null : (Nullable<DateTime>)o;
            }
            set { ViewState["MinDate"] = value; }
        }

        /// <summary>
        /// Gets or sets the Calendar's maximum selectable date
        /// </summary>
        public Nullable<DateTime> MaxDate
        {
            get
            {
                object o = ViewState["MaxDate"];
                return o == null ? null : (Nullable<DateTime>)o;
            }
            set { ViewState["MaxDate"] = value; }
        }

        /// <summary>
        /// Sets the Calendar's title, displayed at the top the the container
        /// </summary>
        public string Title
        {
            get
            {
                object o = ViewState["Title"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Title"] = value; }
        }

        /// <summary>
        /// When set to true, displays a close icon that can be used to dismiss the Calendar
        /// </summary>
        public bool Closable
        {
            get
            {
                object o = ViewState["Closable"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["Closable"] = value; }
        }

        /// <summary>
        /// Places an iframe shim underneath the Calendar to prevent select elements from bleeding through
        /// </summary>
        public bool UseIFrame
        {
            get
            {
                object o = ViewState["UseIFrame"];
                return o == null ? true : (bool)o;
            }
            set { ViewState["UseIFrame"] = value; }
        }

        /// <summary>
        /// Determines whether the Calendar should allow for the selection of multiple dates
        /// </summary>
        public bool MultiSelect
        {
            get
            {
                object o = ViewState["MultiSelect"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["MultiSelect"] = value; }
        }

        /// <summary>
        /// Determines whether to display the weekday headers 
        /// </summary>
        public bool ShowWeekdays
        {
            get
            {
                object o = ViewState["ShowWeekdays"];
                return o == null ? true : (bool)o;
            }
            set { ViewState["ShowWeekdays"] = value; }
        }

        /// <summary>
        /// The format of the month title to be displayed
        /// </summary>
        public MonthFormat MonthFormat
        {
            get
            {
                object o = ViewState["MonthFormat"];
                return o == null ? MonthFormat.Long : (MonthFormat)o;
            }
            set { ViewState["MonthFormat"] = value; }
        }

        /// <summary>
        /// The format of the weekday title to be displayed
        /// </summary>
        public WeekdayFormat WeekdayFormat
        {
            get
            {
                object o = ViewState["WeekdayFormat"];
                return o == null ? WeekdayFormat.Short : (WeekdayFormat)o;
            }
            set { ViewState["WeekdayFormat"] = value; }
        }

        /// <summary>
        /// Gets or sets day that a week begins on
        /// </summary>
        public DayOfWeek WeekStartDay
        {
            get
            {
                object o = ViewState["WeekStartDay"];
                return o == null ? DayOfWeek.Sunday : (DayOfWeek)o;
            }
            set { ViewState["WeekStartDay"] = value; }
        }

        /// <summary>
        /// Determines whether to display row headers 
        /// </summary>
        public bool ShowWeekHeader
        {
            get
            {
                object o = ViewState["ShowWeekHeader"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["ShowWeekHeader"] = value; }
        }

        /// <summary>
        /// Determines whether to display row footers 
        /// </summary>
        public bool ShowWeekFooter
        {
            get
            {
                object o = ViewState["ShowWeekFooter"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["ShowWeekFooter"] = value; }
        }

        /// <summary>
        /// Determines whether to hide extra weeks that are completely outside the current month
        /// </summary>
        public bool HideBlankWeeks
        {
            get
            {
                object o = ViewState["HideBlankWeeks"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["HideBlankWeeks"] = value; }
        }

        /// <summary>
        /// The image path used for the left navigation arrow 
        /// </summary>
        public string NavLeftImageUrl
        {
            get
            {
                object o = ViewState["NavLeftImageUrl"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["NavLeftImageUrl"] = value; }
        }

        /// <summary>
        /// The image path used for the right navigation arrow 
        /// </summary>
        public string NavRightImageUrl
        {
            get
            {
                object o = ViewState["NavRightImageUrl"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["NavRightImageUrl"] = value; }
        }

        public int Pages
        {
            get
            {
                object o = ViewState["Pages"];
                return o == null ? 1 : (int)o;
            }
            set { ViewState["Pages"] = value; }
        }

        #endregion

        #region Overriden Methods

        protected override void Render(HtmlTextWriter writer)
        {
            writer.WriteBeginTag("div");
            HtmlHelper.WriteStyle(ControlStyle, writer);
            writer.WriteAttribute("id", ContainerID);
            foreach (string key in Attributes.Keys)
            {
                writer.WriteAttribute(key, Attributes[key]);
            }
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteEndTag("div");
            writer.WriteBeginTag("input");
            writer.WriteAttribute("type", "hidden");
            writer.WriteAttribute("name", HiddenFieldID);
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteEndTag("input");
            writer.WriteLine();
        }

        #endregion

        #region IYahooControl Members

        public string LoadScript
        {
            get
            {
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = new DelimitedList<string>(", ");
                options.Add(String.Format("pagedate: \"{0:MM/yyyy}\"", PageDate));
                if (MinDate.HasValue)
                    options.Add(String.Format("mindate: \"{0:d}\"", MinDate));
                if (MaxDate.HasValue)
                    options.Add(String.Format("maxdate: \"{0:d}\"", MaxDate));
                if (!String.IsNullOrEmpty(Title))
                    options.Add(String.Format("title: \"{0}\"", Title));
                if (Closable)
                    options.Add(String.Format("close: {0}", Closable.ToString().ToLower()));
                if (!UseIFrame)
                    options.Add(String.Format("iframe : {0}", UseIFrame.ToString().ToLower()));
                if (MultiSelect)
                    options.Add(String.Format("MULTI_SELECT : {0}", MultiSelect.ToString().ToLower()));
                if (!ShowWeekdays)
                    options.Add(String.Format("SHOW_WEEKDAYS : {0}", ShowWeekdays.ToString().ToLower()));
                if (MonthFormat != MonthFormat.Long)
                    options.Add(String.Format("LOCALE_MONTHS: \"{0}\"", EnumHelper.GetEnumDescription(MonthFormat)));
                if (WeekdayFormat != WeekdayFormat.Short)
                    options.Add(String.Format("LOCALE_WEEKDAYS: \"{0}\"", EnumHelper.GetEnumDescription(WeekdayFormat)));
                if (WeekStartDay != DayOfWeek.Sunday)
                    options.Add(String.Format("START_WEEKDAY : {0}", (int)WeekStartDay));
                if (ShowWeekHeader)
                    options.Add(String.Format("SHOW_WEEK_HEADER: {0}", ShowWeekHeader.ToString().ToLower()));
                if (ShowWeekFooter)
                    options.Add(String.Format("SHOW_WEEK_FOOTER: {0}", ShowWeekFooter.ToString().ToLower()));
                if (HideBlankWeeks)
                    options.Add(String.Format("HIDE_BLANK_WEEKS: {0}", HideBlankWeeks.ToString().ToLower()));
                if (!String.IsNullOrEmpty(NavLeftImageUrl))
                    options.Add(String.Format("NAV_ARROW_LEFT: \"{0}\"", NavLeftImageUrl));
                if (!String.IsNullOrEmpty(NavRightImageUrl))
                    options.Add(String.Format("NAV_ARROW_RIGHT: \"{0}\"", NavRightImageUrl));
                if (!String.IsNullOrEmpty(SelectedDateString))
                    options.Add(String.Format("selected: \"{0}\"", SelectedDateString));
                if (Pages > 2)
                    options.Add(String.Format("pages: {0}", Pages));
                string className = Pages > 1 ? "CalendarGroup" : "Calendar";
                s.AppendFormat("{0} = new  YAHOO.widget.{2}(\"{0}\", \"{1}\",", ID, ContainerID, className);
                s.Append(" { ");
                s.Append(options.ToString());
                s.Append("} );");
                s.AppendFormat(" {0}.render();", ID);
                s.AppendFormat("{0}.selectEvent.subscribe(SetHiddenSelectedDates, {0}, true);", ID);
                return s.ToString();
            }
        }

        public YahooScript[] ScriptDependencies
        {
            get { return new YahooScript[] { YahooScript.Yahoo_Dom_Event, YahooScript.Calendar, YahooScript.CalendarSave }; }
        }

        public YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Calendar }; }
        }

        #endregion
    }
}