#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.ComponentModel;

#endregion

namespace Web.YUI
{
    [ParseChildren(true), PersistChildren(false)]
    public class DragDropExtender : YahooControl
    {
        #region Fields

        private EventCollection events = null;

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public EventCollection Events
        {
            get
            {
                if (this.events == null)
                {
                    this.events = new EventCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.events.TrackViewState();
                    }
                }
                return this.events;
            }
        }

        public string ControlID
        {
            get
            {
                object o = ViewState["ControlID"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["ControlID"] = value; }
        }

        public bool UseProxy
        {
            get
            {
                object o = ViewState["UseProxy"];
                return o == null ? false : (bool)o;
            }
            set { ViewState["UseProxy"] = value; }
        }

        public string InteractionGroup
        {
            get
            {
                object o = ViewState["InteractionGroup"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["InteractionGroup"] = value; }
        }

        #region Events

        /// <summary>
        /// Provides access to the mousedown event. The mousedown does not always result in a drag operation.
        /// </summary>
        public string OnMouseDown
        {
            get
            {
                object o = ViewState["OnMouseDown"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnMouseDown"] = value; }
        }

        /// <summary>
        /// Occurs after a mouse down and the drag threshold has been met. The drag threshold default is either 3 pixels of mouse movement or 1 full second of holding the mousedown.
        /// </summary>
        public string OnStartDrag
        {
            get
            {
                object o = ViewState["OnStartDrag"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnStartDrag"] = value; }
        }

        /// <summary>
        /// Occurs every mousemove event while dragging.
        /// </summary>
        public string OnDrag
        {
            get
            {
                object o = ViewState["OnDrag"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnDrag"] = value; }
        }

        /// <summary>
        /// Occurs when the dragged object first interacts with another targettable drag and drop object.
        /// </summary>
        public string OnDragEnter
        {
            get
            {
                object o = ViewState["OnDragEnter"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnDragEnter"] = value; }
        }

        /// <summary>
        /// Fires every mousemove event while over a drag and drop object.
        /// </summary>
        public string OnDragOver
        {
            get
            {
                object o = ViewState["OnDragOver"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnDragOver"] = value; }
        }

        /// <summary>
        /// Fires when a dragged object is no longer over an object that had the onDragEnter fire.
        /// </summary>
        public string OnDragOut
        {
            get
            {
                object o = ViewState["OnDragOut"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnDragOut"] = value; }
        }

        /// <summary>
        /// Fires when the dragged objects is dropped on another.
        /// </summary>
        public string OnDragDrop
        {
            get
            {
                object o = ViewState["OnDragDrop"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnDragDrop"] = value; }
        }

        /// <summary>
        /// Fires when the dragged objects is dropped in a location that contains no drop targets.
        /// </summary>
        public string OnInvalidDrop
        {
            get
            {
                object o = ViewState["OnInvalidDrop"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnInvalidDrop"] = value; }
        }

        /// <summary>
        /// Fires on the mouseup event after a drag has been initiated (startDrag fired).
        /// </summary>
        public string OnEndDrag
        {
            get
            {
                object o = ViewState["OnEndDrag"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnEndDrag"] = value; }
        }

        /// <summary>
        /// Fires on the mouseup event whether or not a drag was initiated. 
        /// </summary>
        public string OnMouseUp
        {
            get
            {
                object o = ViewState["OnMouseUp"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["OnMouseUp"] = value; }
        }

        #endregion

        #endregion

        #region IYahooControl Members

        public override string LoadScript
        {
            get { return String.Empty; }
        }

        public override YahooScript[] ScriptDependencies
        {
            get { return new YahooScript[] { YahooScript.Yahoo_Dom_Event, YahooScript.DragDrop }; }
        }

        public override YahooStyleSheet[] StyleDependencies
        {
            get { return null; }
        }

        #endregion
    }
}