#region Using Directives

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Text;
using System.Collections.Generic;

#endregion

namespace Web.YUI
{
    public class FloatPanel : Overlay
    {
        #region Fields

        private static string HeaderClass = "hd";
        private ITemplate headerTemplate = null;
        private WebControl headerControl;

        private static string BodyClass = "bd";
        private ITemplate bodyTemplate = null;
        private WebControl bodyControl;

        private static string FooterClass = "ft";
        private ITemplate footerTemplate = null;
        private WebControl footerControl;

        #endregion

        #region Properties

        /// <summary>
        /// Whether a "close" icon should be displayed in the header.
        /// </summary>
        public bool Closable
        {
            get
            {
                object o = ViewState["Closable"];
                return o == null ? true : (bool)o;
            }
            set
            {
                ViewState["Closable"] = value;
            }
        }

        public bool Resizable
        {
            get
            {
                object o = ViewState["Resizable"];
                return o == null ? false : (bool)o;
            }
            set
            {
                ViewState["Resizable"] = value;
            }
        }

        /// <summary>
        /// Whether to allow the user to drag the Panel using its header.
        /// </summary>
        public bool Draggable
        {
            get
            {
                object o = ViewState["Draggable"];
                return o == null ? true : (bool)o;
            }
            set
            {
                ViewState["Draggable"] = value;
            }
        }

        /// <summary>
        /// Specifies the type of underlay to display under the Panel. Other options include "none", and "matte", which renders a small white matte under the Panel.
        /// </summary>
        public FloatPanelUnderlay Underlay
        {
            get
            {
                object o = ViewState["Underlay"];
                return o == null ? FloatPanelUnderlay.Shadow : (FloatPanelUnderlay)o;
            }
            set
            {
                ViewState["Underlay"] = value;
            }
        }

        /// <summary>
        /// Specifies whether the document should be shielded with a partially transparent mask to require the user to close the Panel before being able to activate any elements in the document. 
        /// </summary>
        public bool Modal
        {
            get
            {
                object o = ViewState["Modal"];
                return o == null ? false : (bool)o;
            }
            set
            {
                ViewState["Modal"] = value;
            }
        }

        public bool PanelVisible
        {
            get
            {
                object o = ViewState["PanelVisible"];
                return o == null ? true : (bool)o;
            }
            set
            {
                ViewState["PanelVisible"] = value;
            }
        }


        [TemplateInstance(TemplateInstance.Single), MergableProperty(false), PersistenceMode(PersistenceMode.InnerProperty), Browsable(false)]
        public ITemplate HeaderTemplate
        {
            get { return this.headerTemplate; }
            set { this.headerTemplate = value; }
        }

        public WebControl HeaderControl
        {
            get { EnsureChildControls(); return headerControl; }
        }

        [TemplateInstance(TemplateInstance.Single), MergableProperty(false), PersistenceMode(PersistenceMode.InnerProperty), Browsable(false)]
        public ITemplate BodyTemplate
        {
            get { return this.bodyTemplate; }
            set { this.bodyTemplate = value; }
        }

        public WebControl BodyControl
        {
            get { EnsureChildControls(); return bodyControl; }
        }

        [TemplateInstance(TemplateInstance.Single), MergableProperty(false), PersistenceMode(PersistenceMode.InnerProperty), Browsable(false)]
        public ITemplate FooterTemplate
        {
            get { return this.footerTemplate; }
            set { this.footerTemplate = value; }
        }

        public WebControl FooterControl
        {
            get { EnsureChildControls(); return footerControl; }
        }

        #endregion

        #region Overriden Methods

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            headerControl = new System.Web.UI.WebControls.WebControl(HtmlTextWriterTag.Div);
            headerControl.CssClass = HeaderClass;
            this.Controls.Add(headerControl);

            bodyControl = new System.Web.UI.WebControls.WebControl(HtmlTextWriterTag.Div);
            bodyControl.CssClass = BodyClass;
            this.Controls.Add(bodyControl);

            footerControl = new System.Web.UI.WebControls.WebControl(HtmlTextWriterTag.Div);
            footerControl.CssClass = FooterClass;
            this.Controls.Add(footerControl);
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            EnsureChildControls();

            if (headerTemplate != null)
                this.headerTemplate.InstantiateIn(headerControl);

            if (bodyTemplate != null)
                this.bodyTemplate.InstantiateIn(bodyControl);

            if (footerTemplate != null)
                this.footerTemplate.InstantiateIn(footerControl);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            if (Visible)
            {
                writer.WriteBeginTag("div");
                //HtmlHelper.WriteStyle(ControlStyle, writer);
                writer.WriteAttribute("id", ID);
                //foreach (string key in Attributes.Keys)
                //{
                //    writer.WriteAttribute(key, Attributes[key]);
                //}
                writer.Write(HtmlTextWriter.TagRightChar);
                writer.WriteLine();
                writer.Indent++;

                headerControl.RenderControl(writer);
                bodyControl.RenderControl(writer);
                footerControl.RenderControl(writer);

                writer.Indent--;
                writer.WriteEndTag("div");
                writer.WriteLine();
            }
        }

        #endregion

        #region IYahooControl Members

        public override string LoadScript
        {
            get
            {
                if (!Visible)
                    return String.Empty;
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = BuildOptions();
                string panelType = Resizable ? "ResizePanel" : "Panel";
                s.AppendFormat("{0}_panel = new  YAHOO.widget.{1}(\"{0}\",", ID, panelType);
                s.Append(" { ");
                s.Append(options.ToString());
                s.Append("} );");
                s.AppendFormat(" {0}_panel.render();", ID);
                return s.ToString();
            }
        }

        protected override DelimitedList<string> BuildOptions()
        {
            DelimitedList<string> options = base.BuildOptions();
            options.Add(String.Format("close: {0}", Closable.ToString().ToLower()));
            options.Add(String.Format("draggable: {0}", Draggable.ToString().ToLower()));
            options.Add(String.Format("modal: {0}", Modal.ToString().ToLower()));
            options.Add(String.Format("underlay: \"{0}\"", EnumHelper.GetEnumDescription(Underlay)));
            return options;
        }

        public override YahooScript[] ScriptDependencies
        {
            get
            {
                List<YahooScript> scripts = new List<YahooScript>();
                scripts.Add(YahooScript.Yahoo_Dom_Event);
                scripts.Add(YahooScript.Connection);
                scripts.Add(YahooScript.Animation);
                scripts.Add(YahooScript.Container);

                if (Resizable || Draggable)
                    scripts.Add(YahooScript.DragDrop);

                if (Resizable)
                    scripts.Add(YahooScript.ResizePanel);

                foreach (Animation a in Animations)
                {
                    if (a.Type == AnimationType.HorizontalExpand ||
                        a.Type == AnimationType.VerticalBodyExpand ||
                        a.Type == AnimationType.VerticalExpand)
                    {
                        scripts.Add(YahooScript.PanelEffects);
                    }
                }
                return scripts.ToArray();
            }
        }

        public override YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Container }; }
        }

        #endregion
    }
}