using System;
using System.Collections.Generic;
using System.Web.UI;

namespace Web.YUI
{
    public abstract class MenuBaseBuilder 
    {
        private const int StartMenuLevel = 0;

        private Dictionary<String, String> cssClasses = new Dictionary<string, string>();

        private HtmlTextWriter writer;

        private MenuItemCollection items;

        private String menuId;

        private Page page;

        public virtual Dictionary<string, string> CssClasses
        {
            get { return cssClasses; }
        }

        public virtual HtmlTextWriter Writer
        {
            get
            {
                 return writer;
            }
            set
            {
                 writer = value;
            }
        }

        public virtual string MenuId
        {
            get { return menuId; }
            set { menuId = value; }
        }

        public virtual MenuItemCollection Items
        {
            get { return items; }
            set { items = value; }
        }

        public Page Page
        {
            get { return page; }
            set { page = value; }
        }

        protected MenuBaseBuilder()
        {
            InitialiseCssClasses();
        }

        protected virtual void InitialiseCssClasses()
        {
            CssClasses["yuimenu"] = "yuimenu";
            CssClasses["yuimenu.container"] = "yuimenu";
            CssClasses["bd"] = "bd";
            CssClasses["first-of-type"] = "first-of-type";
            CssClasses["yui-menu-shadow"] = "yui-menu-shadow";
            CssClasses["yuimenuitem.zerolevel"] = "yuimenuitem";
            CssClasses["yuimenuitem"] = "yuimenuitem";
            CssClasses["yuimenuitemlabel.zerolevel"] = "yuimenuitemlabel";
            CssClasses["yuimenuitemlabel"] = "yuimenuitemlabel";
        }

        public virtual void BuildMenu()
        {
            writer.WriteBeginTag("div");
            writer.WriteAttribute("id", MenuId);
            writer.WriteAttribute("class", CssClasses["yuimenu.container"]);
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.Indent++;
            writer.WriteLine();
            writer.Indent++;

            writer.WriteBeginTag("div");
            writer.WriteAttribute("class", CssClasses["bd"]);
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteLine();
            writer.Indent++;

            writer.Indent++;
            BuildItems(Items, true, StartMenuLevel);
            writer.Indent--;
            writer.WriteLine();

            writer.Indent--;
            writer.WriteEndTag("div");
            writer.WriteLine();
            writer.Indent--;
            writer.WriteEndTag("div");            
        }

        protected virtual void BuildItems(MenuItemCollection items, bool isRoot, int level)
        {
            if (items.Count > 0)
            {
                if (!isRoot)
                {
                    writer.WriteBeginTag("div");
                    writer.WriteAttribute("class", CssClasses["yuimenu"]);
                    writer.Write(HtmlTextWriter.TagRightChar);
                    writer.Indent++;
                    writer.WriteLine();
                    writer.Indent++;

                    writer.WriteBeginTag("div");
                    writer.WriteAttribute("class", CssClasses["bd"]);
                    writer.Write(HtmlTextWriter.TagRightChar);
                    writer.WriteLine();
                    writer.Indent++;
                }

                writer.WriteLine();

                writer.WriteBeginTag("ul");
                if (level == StartMenuLevel)
                {
                    writer.WriteAttribute("class", CssClasses["first-of-type"]);
                }
                writer.Write(HtmlTextWriter.TagRightChar);
                writer.Indent++;

                foreach (MenuItem item in items)
                {
                    BuildItem(item, level);
                }

                writer.Indent--;
                writer.WriteLine();
                writer.WriteEndTag("ul");

                if (!isRoot)
                {
                    writer.Indent--;
                    writer.WriteEndTag("div");
                    writer.WriteLine();
                    writer.Indent--;
                    writer.WriteEndTag("div");
                }
            }


        }

        protected virtual void BuildItem(MenuItem item, int level)
        {
            if ((item != null))
            {
                writer.WriteLine();
                writer.WriteBeginTag("li");
                writer.WriteAttribute("class", level == StartMenuLevel ? CssClasses["yuimenuitem.zerolevel"] : CssClasses["yuimenuitem"]);
                writer.Write(HtmlTextWriter.TagRightChar);
                writer.Indent++;

                if (IsLink(item))
                {
                    writer.WriteBeginTag("a");
                    writer.WriteAttribute("class", level == StartMenuLevel ? CssClasses["yuimenuitemlabel.zerolevel"] : CssClasses["yuimenuitemlabel"]);
                    if (!String.IsNullOrEmpty(item.NavigateUrl))
                    {
                        writer.WriteAttribute("href", Page.Server.HtmlEncode(Page.ResolveClientUrl(item.NavigateUrl)));
                    }

                    if (!String.IsNullOrEmpty(item.ToolTip))
                    {
                        writer.WriteAttribute("title", item.ToolTip);
                    }
                    writer.Write(HtmlTextWriter.TagRightChar);
                    writer.Indent++;
                }
                else
                {
                    writer.WriteBeginTag("span");
                    writer.Write(HtmlTextWriter.TagRightChar);
                    writer.Indent++;
                    writer.WriteLine();
                }

                writer.Write(item.Text);

                if (IsLink(item))
                {
                    writer.Indent--;
                    writer.WriteEndTag("a");
                }
                else
                {
                    writer.Indent--;
                    writer.WriteEndTag("span");
                }

                if ((item.ChildItems != null) && (item.ChildItems.Count > 0))
                {
                    BuildItems(item.ChildItems, false, ++level);
                }

                writer.Indent--;
                writer.WriteEndTag("li");
            }
        }

        protected virtual bool IsLink(MenuItem item)
        {
            return (item != null) && item.Enabled && !String.IsNullOrEmpty(item.NavigateUrl);
        }
    }
}
