#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.HtmlControls;

#endregion

namespace Web.YUI
{
    public class Menu : Overlay
    {
        #region Fields

        MenuItemCollection items = null;

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public MenuItemCollection Items
        {
            get
            {
                if (this.items == null)
                {
                    this.items = new MenuItemCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.items.TrackViewState();
                    }
                }
                return this.items;
            }
        }

        /// <summary>
        /// Boolean indicating if submenus are automatically made visible when the user mouses over the menu's items.
        /// </summary>
        public bool AutoSubMenuDisplay
        {
            get
            {
                object o = ViewState["AutoSubMenuDisplay"];
                return o == null ? true : (bool)o;
            }
            set { ViewState["AutoSubMenuDisplay"] = value; }
        }

        /// <summary>
        /// Number indicating the time (in milliseconds) that should expire before a submenu is made visible when the user mouses over the menu's items.
        /// </summary>
        public int ShowDelay
        {
            get
            {
                object o = ViewState["ShowDelay"];
                return o == null ? 0 : (int)o;
            }
            set { ViewState["ShowDelay"] = value; }
        }

        /// <summary>
        /// Number indicating the time (in milliseconds) that should expire before the menu is hidden.
        /// </summary>
        public int HideDelay
        {
            get
            {
                object o = ViewState["HideDelay"];
                return o == null ? 0 : (int)o;
            }
            set { ViewState["HideDelay"] = value; }
        }

        /// <summary>
        /// Boolean indicating if the menu will automatically be hidden if the user clicks outside of it.
        /// </summary>
        public bool ClickToHide
        {
            get
            {
                object o = ViewState["ClickToHide"];
                return o == null ? true : (bool)o;
            }
            set
            {
                ViewState["ClickToHide"] = value;
            }
        }

        /// <summary>
        /// Number indicating the time (in milliseconds) that should expire before a submenu is hidden when the user mouses out of a menu item heading in the direction of a submenu. The value must be greater than or equal to the value specified for the "showdelay" configuration property.
        /// </summary>
        public int SubMenuHideDelay
        {
            get
            {
                object o = ViewState["SubMenuHideDelay"];
                return o == null ? 250 : (int)o;
            }
            set { ViewState["SubMenuHideDelay"] = value; }
        }

        /// <summary>
        /// Defines the maximum height (in pixels) for a menu before the contents of the body are scrolled.
        /// </summary>
        public int MaxHeight
        {
            get
            {
                object o = ViewState["MaxHeight"];
                return o == null ? 0 : (int)o;
            }
            set { ViewState["MaxHeight"] = value; }
        }

        /// <summary>
        /// Defines the maximum height (in pixels) for a menu before the contents of the body are scrolled.
        /// </summary>
        [DefaultValue(YUI.MenuType.TopNavigation), Browsable(true)]
        public MenuType MenuType
        {
            get
            {
                object o = ViewState["MenuType"];
                return o == null ? YUI.MenuType.TopNavigation : (YUI.MenuType)o;
            }
            set
            {
                ViewState["MenuType"] = value;
            }
        }


        #endregion

        #region IYahooControl Members

        public override string LoadScript
        {
            get
            {
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = BuildOptions();
                String format = @"{0} = new YAHOO.widget.{4}(""{0}"", {2} {1} {3}); {0}.render();";
                s.AppendFormat(format, ID, options, "{", "}", MenuType == MenuType.TopNavigation ? "MenuBar" : "Menu");
                return s.ToString();
            }
        }

        public override YahooScript[] ScriptDependencies
        {
            get { return new YahooScript[] { YahooScript.Yahoo_Dom_Event, YahooScript.Container, YahooScript.Menu }; }
        }

        public override YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Fonts, YahooStyleSheet.Menu }; }
        }

        #endregion

        #region Build Options

        protected override DelimitedList<string> BuildOptions()
        {
            DelimitedList<string> options = base.BuildOptions();
            if (ShowDelay > 0)
                options.Add(String.Format("showdelay: {0}", ShowDelay));
            if (HideDelay > 0)
                options.Add(String.Format("hidedelay: {0}", HideDelay));
            if (!ClickToHide)
                options.Add(String.Format("clicktohide: {0}", ClickToHide.ToString().ToLower()));
            if (SubMenuHideDelay != 250)
                options.Add(String.Format("submenuhidedelay: {0}", SubMenuHideDelay));
            if (MaxHeight > 0)
                options.Add(String.Format("maxheight: {0}", MaxHeight));
            
            options.Add(String.Format("autosubmenudisplay: {0}", AutoSubMenuDisplay.ToString().ToLower()));
            RemoveOption("visible", options);
            options.Add(String.Format("visible: {0}", Visible.ToString().ToLower()));
            return options;
        }

        #endregion

        #region Render

        protected override void Render(HtmlTextWriter writer)
        {
            MenuBaseBuilder menuBuilder = MenuBuilderFactory.GetMenuBuilder(this.MenuType);
            menuBuilder.Items = Items;
            menuBuilder.MenuId = this.ID;
            menuBuilder.Page = this.Page;
            menuBuilder.Writer = writer;
            menuBuilder.BuildMenu();
        }

        #endregion
    }
}
