#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

#endregion

namespace Web.YUI
{
    /// <summary>
    /// The Module control enables you to create a JavaScript object representation of a basic module of content. It can be used to manipulate existing content modules on your page or to create modules dynamically and append them to the DOM. All other components in the Container family have Module as their lowest-level base class.
    /// </summary>
    public abstract class Module : YahooControl
    {
        #region Properties

        /// <summary>
        /// Configures whether or not to create a hidden off-screen element that can be used to monitor for text size changes in the DOM. 
        /// </summary>
        public bool MonitorResize
        {
            get
            {
                object o = ViewState["MonitorResize"];
                return o == null ? true : (bool)o;
            }
            set
            {
                ViewState["MonitorResize"] = value;
            }
        }

        #endregion

        #region Build Options

        protected override DelimitedList<string> BuildOptions()
        {
            DelimitedList<string> options = base.BuildOptions();
            if (!Visible)
                options.Add("visible: false");
            if(!MonitorResize)
                options.Add("monitorresize : false");
            return options;
        }

        protected bool HasOption(string name, DelimitedList<string> options)
        {
            Regex r = new Regex("(\\w+):\\s+(\\w+)");
            foreach (string option in options)
            {
                Match m = r.Match(name);
                if (m.Success && m.Groups[1].Value.Equals(name, StringComparison.CurrentCultureIgnoreCase))
                {
                    return true;
                }
            }
            return false;
        }

        protected void RemoveOption(string name, DelimitedList<string> options)
        {
            int index = 0;
            int foundIndex = -1;
            Regex r = new Regex("(\\w+):\\s+(\\w+)");
            foreach (string option in options)
            {
                Match m = r.Match(name);
                if (m.Success && m.Groups[1].Value.Equals(name, StringComparison.CurrentCultureIgnoreCase))
                {
                    foundIndex = index;
                }
                ++index;
            }
            if (foundIndex != -1)
                options.RemoveAt(foundIndex);
        }

        #endregion
    }
}