#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.ComponentModel;

#endregion

namespace Web.YUI
{
    [ParseChildren(true), PersistChildren(false)]
    public class TabView : CompositeControl, IYahooControl
    {
        #region Fields

        private TabViewItemCollection items = null;

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public TabViewItemCollection Items
        {
            get
            {
                if (this.items == null)
                {
                    this.items = new TabViewItemCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.items.TrackViewState();
                    }
                }
                return this.items;
            }
        }

        #endregion

        #region Methods

        protected override void CreateChildControls()
        {
            base.CreateChildControls();
            foreach (TabViewItem tab in Items)
            {
                Controls.Add(tab);
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            writer.WriteBeginTag("div");
            writer.WriteAttribute("id", ID);
            writer.WriteAttribute("class", "yui-navset");
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteLine();
            writer.Indent++;

            writer.WriteBeginTag("ul");
            writer.WriteAttribute("class", "yui-nav");
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteLine();
            writer.Indent++;

            foreach (TabViewItem tab in Items)
            {
                writer.WriteBeginTag("li");
                writer.Write(HtmlTextWriter.TagRightChar);
                writer.Write(tab.Label);
                writer.WriteEndTag("li");
                writer.WriteLine();
            }

            writer.Indent--;
            writer.WriteEndTag("ul");
            writer.WriteLine();

            writer.WriteBeginTag("div");
            writer.WriteAttribute("class", "yui-content");
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.WriteLine();
            writer.Indent++;

            foreach (TabViewItem tab in Items)
            {
                //writer.WriteBeginTag("div");
                //writer.Write(HtmlTextWriter.TagRightChar);

                tab.RenderControl(writer);

                //writer.WriteEndTag("div");
                //writer.WriteLine();
            }

            writer.Indent--;
            writer.WriteEndTag("div");
            writer.WriteLine();

            writer.WriteEndTag("div");
            writer.WriteLine();
            writer.Indent--;
        }

        #endregion

        #region IYahooControl Members

        public string LoadScript
        {
            get
            {
                return String.Format("var {0} = new YAHOO.widget.TabView(\"{0}\");", ID);
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = new DelimitedList<string>(", ");
                return s.ToString();
            }
        }

        public YahooScript[] ScriptDependencies
        {
            get { return new YahooScript[] { YahooScript.Yahoo_Dom_Event, YahooScript.Element, YahooScript.Connection, YahooScript.TabView }; }
        }

        public YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.TabView, YahooStyleSheet.BorderTabs }; }
        }

        #endregion
    }
}