#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI;

#endregion

namespace Web.YUI
{
    /// <summary>
    /// The Tooltip control is an extension of Overlay that is analogous to popup tooltips within common operating systems. The standard tooltip interaction pattern involves a small overlay that is displayed when the mouse hovers over a context element for a specified amount of time. Tooltip is designed to be simple to implement with easily-accessed configuration options and visual styling handled entirely via CSS.
    /// </summary>
    public class Tooltip : Overlay
    {
        #region Constructor

        public Tooltip()
            : base()
        {
            Visible = false;
            ConstrainToViewport = true;
        }

        #endregion

        #region Properties

        public string ContextID
        {
            get
            {
                object o = ViewState["ContextID"];
                return o == null ? String.Empty : (string)o;
            }
            set
            {
                ViewState["ContextID"] = value;
            }
        }

        /// <summary>
        /// Specifies the Tooltip's text.
        /// </summary>
        public string Text
        {
            get
            {
                object o = ViewState["Text"];
                return o == null ? String.Empty : (string)o;
            }
            set { ViewState["Text"] = value; }
        }

        /// <summary>
        /// Specifies whether the Tooltip should be kept from overlapping its context element.
        /// </summary>
        public bool PreventOverlap
        {
            get
            {
                object o = ViewState["PreventOverlap"];
                return o == null ? true : (bool)o;
            }
            set { ViewState["PreventOverlap"] = value; }
        }

        /// <summary>
        /// The number of milliseconds to wait before showing a Tooltip on mouseover.
        /// </summary>
        public int ShowDelay
        {
            get
            {
                object o = ViewState["ShowDelay"];
                return o == null ? 200 : (int)o;
            }
            set { ViewState["ShowDelay"] = value; }
        }

        /// <summary>
        /// The number of milliseconds to wait before hiding a Tooltip on mouseover.
        /// </summary>
        public int HideDelay
        {
            get
            {
                object o = ViewState["HideDelay"];
                return o == null ? 250 : (int)o;
            }
            set { ViewState["HideDelay"] = value; }
        }

        /// <summary>
        /// The number of milliseconds to wait before automatically dismissing a Tooltip after the mouse has been resting on the context element.
        /// </summary>
        public int AutoDismissDelay
        {
            get
            {
                object o = ViewState["AutoDismissDelay"];
                return o == null ? 5000 : (int)o;
            }
            set { ViewState["AutoDismissDelay"] = value; }
        }

        #endregion

        #region IYahooControl Members

        public override string LoadScript
        {
            get
            {
                StringBuilder s = new StringBuilder();
                DelimitedList<string> options = BuildOptions();
                s.AppendFormat("{0} = new  YAHOO.widget.Tooltip(\"{0}\",", ID);
                s.Append(" { ");
                s.Append(options.ToString());
                s.Append("} );");
                return s.ToString();
            }
        }

        public override YahooScript[] ScriptDependencies
        {
            get
            {
                List<YahooScript> scripts = new List<YahooScript>();
                scripts.Add(YahooScript.Yahoo_Dom_Event);
                scripts.Add(YahooScript.Animation);
                scripts.Add(YahooScript.Container);

                foreach (Animation a in Animations)
                {
                    if (a.Type == AnimationType.HorizontalExpand ||
                        a.Type == AnimationType.VerticalBodyExpand ||
                        a.Type == AnimationType.VerticalExpand)
                    {
                        scripts.Add(YahooScript.PanelEffects);
                    }
                }
                return scripts.ToArray();
            }
        }

        public override YahooStyleSheet[] StyleDependencies
        {
            get { return new YahooStyleSheet[] { YahooStyleSheet.Container }; }
        }

        #endregion

        #region Build Options

        protected override DelimitedList<string> BuildOptions()
        {
            DelimitedList<string> options = base.BuildOptions();
            if (String.IsNullOrEmpty(ContextID))
                throw new ArgumentNullException("ContextID");
            options.Add(String.Format("context: \"{0}\"", ContextID));
            if (!String.IsNullOrEmpty(Text))
                options.Add(String.Format("text: \"{0}\"", Text));
            if (!PreventOverlap)
                options.Add("preventoverlap: false");
            if (ShowDelay != 200)
                options.Add(String.Format("showdelay: {0}", ShowDelay));
            if (HideDelay != 250)
                options.Add(String.Format("hidedelay: {0}", HideDelay));
            if (AutoDismissDelay != 5000)
                options.Add(String.Format("autodismissdelay: {0}", AutoDismissDelay));
            return options;
        }

        #endregion
    }
}