using System;
using System.Configuration;

namespace Web
{
	/// <summary>
	/// Helper class makes it easy to get values from AppSettings.
	/// </summary>
	public static class ConfigHelper
    {
        #region GetString

        public static string GetString(string key)
		{
			return GetString(key, String.Empty);
		}

		public static string GetString(string key, string defaultValue)
		{
            string val = ConfigurationManager.AppSettings[key];
			if(val == null && val != String.Empty)
			{
				return defaultValue;
			}
			else
			{
				return val;
			}
        }

        #endregion

        #region GetInt32

        public static int? GetInt32(string key)
		{
			return GetInt32(key, null);
		}

		public static int? GetInt32(string key, int? defaultValue)
		{
            string val = ConfigurationManager.AppSettings[key];
			if(String.IsNullOrEmpty(val))
			{
				return defaultValue;
			}
			else
			{
                int result;
                if (Int32.TryParse(val, out result))
                {
                    return new Nullable<int>(result);
                }
                return defaultValue;
			}
        }

        #endregion

        #region GetDouble

        public static double? GetDouble(string key)
		{
			return GetDouble(key, null);
		}

		public static double? GetDouble(string key, double? defaultValue)
		{
            string val = ConfigurationManager.AppSettings[key];
			if(String.IsNullOrEmpty(val))
			{
				return defaultValue ;
			}
			else
			{
                double result;
                if (Double.TryParse(val, out result))
                {
                    return new Nullable<double>(result);
                }
                return defaultValue;
			}
        }

        #endregion

        #region GetBoolean

        public static bool? GetBoolean(string key)
		{
			return GetBoolean(key, null);
		}

		public static bool? GetBoolean(string key, bool? defaultValue)
		{
            string val = ConfigurationManager.AppSettings[key];
			if(String.IsNullOrEmpty(val))
			{
				return defaultValue ;
			}
			else
			{
                bool result;
                if (Boolean.TryParse(val, out result))
                {
                    return new Nullable<bool>(result);
                }
                return defaultValue;
			}
        }

        #endregion

        #region GetDateTime

        public static DateTime? GetDateTime(string key)
        {
            return GetDateTime(key, null);
        }

        public static DateTime? GetDateTime(string key, DateTime? defaultValue)
        {
            string val = ConfigurationManager.AppSettings[key];
            if (String.IsNullOrEmpty(val))
            {
                return defaultValue;
            }
            else
            {
                DateTime result;
                if (DateTime.TryParse(val, out result))
                {
                    return new Nullable<DateTime>(result);
                }
                return defaultValue;
            }
        }

        #endregion

        #region GetEnum

        public static T GetEnum<T>(string key, T defaultValue)
        {
            T o;
            string val = ConfigurationManager.AppSettings[key];
            if (String.IsNullOrEmpty(val))
                return defaultValue;

            try
            {
                o = (T)Enum.Parse(typeof(T), val, true);
            }
            catch (FormatException)
            {
                return defaultValue;
            }
            return o;
        }

        #endregion
    }
}