#region Using Directives

using System;
using System.Configuration;
using System.Web;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.Web.UI;

#endregion

namespace Web
{
    public delegate void FindControlDelegate(Control control);

    public static class ControlHelper
    {
        #region Recurse Controls

        /// <summary>
        /// Traverses a control tree and calls the given FindControlDelegate
        /// for each control that is found.
        /// </summary>
        /// <param name="controls">The root of the control tree you want to search</param>
        /// <param name="callback">The method to call for each control that is found</param>
        public static void RecurseControls(ControlCollection controls, FindControlDelegate callback)
        {
            foreach (Control c in controls)
            {
                callback(c);
                if (c.Controls.Count > 0)
                {
                    RecurseControls(c.Controls, callback);
                }
            }
        }

        #endregion

        #region Find Controls

        /// <summary>
        /// Traverses through the control tree using the given ControllCollection
        /// as the root and finds all which match the given type T.
        /// </summary>
        /// <typeparam name="T">The Type to match</typeparam>
        /// <param name="controls">The root collection to start from</param>
        /// <returns></returns>
        public static List<T> FindAllControls<T>(ControlCollection controls)
            where T : Control
        {
            List<T> foundControls = new List<T>();
            RecurseControls(controls, delegate(Control c)
            {
                if(c is T)
                    foundControls.Add(c as T);
            });
            return foundControls;
        }

        #endregion
    }
}