#region Using Directives

using System;
using System.Web.Security;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Text;
using System.Text.RegularExpressions;
using System.Drawing;
using System.IO;
using System.Web;
using System.Collections;
using System.Globalization;

#endregion

namespace Web
{
	public sealed class HtmlHelper
	{
		#region Member Variables

		private static Hashtable attrKeyLookupTable;
		private static AttributeInformation[] attrNameLookupArray;
		private static Hashtable tagKeyLookupTable;
		private static TagInformation[] tagNameLookupArray;
		private static string[] styleNameLookupArray;
		private static Hashtable styleKeyLookupTable;
 
		#endregion

		#region Constructors

		static HtmlHelper()
		{
			tagKeyLookupTable = new Hashtable(97);
			tagNameLookupArray = new TagInformation[97];
			RegisterTag("", HtmlTextWriterTag.Unknown, TagType.Other);
			RegisterTag("a", HtmlTextWriterTag.A, TagType.Inline);
			RegisterTag("acronym", HtmlTextWriterTag.Acronym, TagType.Inline);
			RegisterTag("address", HtmlTextWriterTag.Address, TagType.Other);
			RegisterTag("area", HtmlTextWriterTag.Area, TagType.Other);
			RegisterTag("b", HtmlTextWriterTag.B, TagType.Inline);
			RegisterTag("base", HtmlTextWriterTag.Base, TagType.NonClosing);
			RegisterTag("basefont", HtmlTextWriterTag.Basefont, TagType.NonClosing);
			RegisterTag("bdo", HtmlTextWriterTag.Bdo, TagType.Inline);
			RegisterTag("bgsound", HtmlTextWriterTag.Bgsound, TagType.NonClosing);
			RegisterTag("big", HtmlTextWriterTag.Big, TagType.Inline);
			RegisterTag("blockquote", HtmlTextWriterTag.Blockquote, TagType.Other);
			RegisterTag("body", HtmlTextWriterTag.Body, TagType.Other);
			RegisterTag("br", HtmlTextWriterTag.Br, TagType.Other);
			RegisterTag("button", HtmlTextWriterTag.Button, TagType.Inline);
			RegisterTag("caption", HtmlTextWriterTag.Caption, TagType.Other);
			RegisterTag("center", HtmlTextWriterTag.Center, TagType.Other);
			RegisterTag("cite", HtmlTextWriterTag.Cite, TagType.Inline);
			RegisterTag("code", HtmlTextWriterTag.Code, TagType.Inline);
			RegisterTag("col", HtmlTextWriterTag.Col, TagType.NonClosing);
			RegisterTag("colgroup", HtmlTextWriterTag.Colgroup, TagType.Other);
			RegisterTag("del", HtmlTextWriterTag.Del, TagType.Inline);
			RegisterTag("dd", HtmlTextWriterTag.Dd, TagType.Inline);
			RegisterTag("dfn", HtmlTextWriterTag.Dfn, TagType.Inline);
			RegisterTag("dir", HtmlTextWriterTag.Dir, TagType.Other);
			RegisterTag("div", HtmlTextWriterTag.Div, TagType.Other);
			RegisterTag("dl", HtmlTextWriterTag.Dl, TagType.Other);
			RegisterTag("dt", HtmlTextWriterTag.Dt, TagType.Inline);
			RegisterTag("em", HtmlTextWriterTag.Em, TagType.Inline);
			RegisterTag("embed", HtmlTextWriterTag.Embed, TagType.NonClosing);
			RegisterTag("fieldset", HtmlTextWriterTag.Fieldset, TagType.Other);
			RegisterTag("font", HtmlTextWriterTag.Font, TagType.Inline);
			RegisterTag("form", HtmlTextWriterTag.Form, TagType.Other);
			RegisterTag("frame", HtmlTextWriterTag.Frame, TagType.NonClosing);
			RegisterTag("frameset", HtmlTextWriterTag.Frameset, TagType.Other);
			RegisterTag("h1", HtmlTextWriterTag.H1, TagType.Other);
			RegisterTag("h2", HtmlTextWriterTag.H2, TagType.Other);
			RegisterTag("h3", HtmlTextWriterTag.H3, TagType.Other);
			RegisterTag("h4", HtmlTextWriterTag.H4, TagType.Other);
			RegisterTag("h5", HtmlTextWriterTag.H5, TagType.Other);
			RegisterTag("h6", HtmlTextWriterTag.H6, TagType.Other);
			RegisterTag("head", HtmlTextWriterTag.Head, TagType.Other);
			RegisterTag("hr", HtmlTextWriterTag.Hr, TagType.NonClosing);
			RegisterTag("html", HtmlTextWriterTag.Html, TagType.Other);
			RegisterTag("i", HtmlTextWriterTag.I, TagType.Inline);
			RegisterTag("iframe", HtmlTextWriterTag.Iframe, TagType.Other);
			RegisterTag("img", HtmlTextWriterTag.Img, TagType.NonClosing);
			RegisterTag("input", HtmlTextWriterTag.Input, TagType.NonClosing);
			RegisterTag("ins", HtmlTextWriterTag.Ins, TagType.Inline);
			RegisterTag("isindex", HtmlTextWriterTag.Isindex, TagType.NonClosing);
			RegisterTag("kbd", HtmlTextWriterTag.Kbd, TagType.Inline);
			RegisterTag("label", HtmlTextWriterTag.Label, TagType.Inline);
			RegisterTag("legend", HtmlTextWriterTag.Legend, TagType.Other);
			RegisterTag("li", HtmlTextWriterTag.Li, TagType.Inline);
			RegisterTag("link", HtmlTextWriterTag.Link, TagType.NonClosing);
			RegisterTag("map", HtmlTextWriterTag.Map, TagType.Other);
			RegisterTag("marquee", HtmlTextWriterTag.Marquee, TagType.Other);
			RegisterTag("menu", HtmlTextWriterTag.Menu, TagType.Other);
			RegisterTag("meta", HtmlTextWriterTag.Meta, TagType.NonClosing);
			RegisterTag("nobr", HtmlTextWriterTag.Nobr, TagType.Inline);
			RegisterTag("noframes", HtmlTextWriterTag.Noframes, TagType.Other);
			RegisterTag("noscript", HtmlTextWriterTag.Noscript, TagType.Other);
			RegisterTag("object", HtmlTextWriterTag.Object, TagType.Other);
			RegisterTag("ol", HtmlTextWriterTag.Ol, TagType.Other);
			RegisterTag("option", HtmlTextWriterTag.Option, TagType.Other);
			RegisterTag("p", HtmlTextWriterTag.P, TagType.Inline);
			RegisterTag("param", HtmlTextWriterTag.Param, TagType.Other);
			RegisterTag("pre", HtmlTextWriterTag.Pre, TagType.Other);
			RegisterTag("ruby", HtmlTextWriterTag.Ruby, TagType.Other);
			RegisterTag("rt", HtmlTextWriterTag.Rt, TagType.Other);
			RegisterTag("q", HtmlTextWriterTag.Q, TagType.Inline);
			RegisterTag("s", HtmlTextWriterTag.S, TagType.Inline);
			RegisterTag("samp", HtmlTextWriterTag.Samp, TagType.Inline);
			RegisterTag("script", HtmlTextWriterTag.Script, TagType.Other);
			RegisterTag("select", HtmlTextWriterTag.Select, TagType.Other);
			RegisterTag("small", HtmlTextWriterTag.Small, TagType.Other);
			RegisterTag("span", HtmlTextWriterTag.Span, TagType.Inline);
			RegisterTag("strike", HtmlTextWriterTag.Strike, TagType.Inline);
			RegisterTag("strong", HtmlTextWriterTag.Strong, TagType.Inline);
			RegisterTag("style", HtmlTextWriterTag.Style, TagType.Other);
			RegisterTag("sub", HtmlTextWriterTag.Sub, TagType.Inline);
			RegisterTag("sup", HtmlTextWriterTag.Sup, TagType.Inline);
			RegisterTag("table", HtmlTextWriterTag.Table, TagType.Other);
			RegisterTag("tbody", HtmlTextWriterTag.Tbody, TagType.Other);
			RegisterTag("td", HtmlTextWriterTag.Td, TagType.Inline);
			RegisterTag("textarea", HtmlTextWriterTag.Textarea, TagType.Inline);
			RegisterTag("tfoot", HtmlTextWriterTag.Tfoot, TagType.Other);
			RegisterTag("th", HtmlTextWriterTag.Th, TagType.Inline);
			RegisterTag("thead", HtmlTextWriterTag.Thead, TagType.Other);
			RegisterTag("title", HtmlTextWriterTag.Title, TagType.Other);
			RegisterTag("tr", HtmlTextWriterTag.Tr, TagType.Other);
			RegisterTag("tt", HtmlTextWriterTag.Tt, TagType.Inline);
			RegisterTag("u", HtmlTextWriterTag.U, TagType.Inline);
			RegisterTag("ul", HtmlTextWriterTag.Ul, TagType.Other);
			RegisterTag("var", HtmlTextWriterTag.Var, TagType.Inline);
			RegisterTag("wbr", HtmlTextWriterTag.Wbr, TagType.NonClosing);
			RegisterTag("xml", HtmlTextWriterTag.Xml, TagType.Other);
			attrKeyLookupTable = new Hashtable(40);
			attrNameLookupArray = new AttributeInformation[40];
			RegisterAttribute("accesskey", HtmlTextWriterAttribute.Accesskey, true);
			RegisterAttribute("align", HtmlTextWriterAttribute.Align, false);
			RegisterAttribute("alt", HtmlTextWriterAttribute.Alt, true);
			RegisterAttribute("background", HtmlTextWriterAttribute.Background, true);
			RegisterAttribute("bgcolor", HtmlTextWriterAttribute.Bgcolor, false);
			RegisterAttribute("border", HtmlTextWriterAttribute.Border, false);
			RegisterAttribute("bordercolor", HtmlTextWriterAttribute.Bordercolor, false);
			RegisterAttribute("cellpadding", HtmlTextWriterAttribute.Cellpadding, false);
			RegisterAttribute("cellspacing", HtmlTextWriterAttribute.Cellspacing, false);
			RegisterAttribute("checked", HtmlTextWriterAttribute.Checked, false);
			RegisterAttribute("class", HtmlTextWriterAttribute.Class, true);
			RegisterAttribute("cols", HtmlTextWriterAttribute.Cols, false);
			RegisterAttribute("colspan", HtmlTextWriterAttribute.Colspan, false);
			RegisterAttribute("disabled", HtmlTextWriterAttribute.Disabled, false);
			RegisterAttribute("for", HtmlTextWriterAttribute.For, false);
			RegisterAttribute("height", HtmlTextWriterAttribute.Height, false);
			RegisterAttribute("href", HtmlTextWriterAttribute.Href, true);
			RegisterAttribute("id", HtmlTextWriterAttribute.Id, false);
			RegisterAttribute("maxlength", HtmlTextWriterAttribute.Maxlength, false);
			RegisterAttribute("multiple", HtmlTextWriterAttribute.Multiple, false);
			RegisterAttribute("name", HtmlTextWriterAttribute.Name, false);
			RegisterAttribute("nowrap", HtmlTextWriterAttribute.Nowrap, false);
			RegisterAttribute("onclick", HtmlTextWriterAttribute.Onclick, true);
			RegisterAttribute("onchange", HtmlTextWriterAttribute.Onchange, true);
			RegisterAttribute("readonly", HtmlTextWriterAttribute.ReadOnly, false);
			RegisterAttribute("rows", HtmlTextWriterAttribute.Rows, false);
			RegisterAttribute("rowspan", HtmlTextWriterAttribute.Rowspan, false);
			RegisterAttribute("rules", HtmlTextWriterAttribute.Rules, false);
			RegisterAttribute("selected", HtmlTextWriterAttribute.Selected, false);
			RegisterAttribute("size", HtmlTextWriterAttribute.Size, false);
			RegisterAttribute("src", HtmlTextWriterAttribute.Src, true);
			RegisterAttribute("style", HtmlTextWriterAttribute.Style, false);
			RegisterAttribute("tabindex", HtmlTextWriterAttribute.Tabindex, false);
			RegisterAttribute("target", HtmlTextWriterAttribute.Target, false);
			RegisterAttribute("title", HtmlTextWriterAttribute.Title, true);
			RegisterAttribute("type", HtmlTextWriterAttribute.Type, false);
			RegisterAttribute("valign", HtmlTextWriterAttribute.Valign, false);
			RegisterAttribute("value", HtmlTextWriterAttribute.Value, true);
			RegisterAttribute("width", HtmlTextWriterAttribute.Width, false);
			RegisterAttribute("wrap", HtmlTextWriterAttribute.Wrap, false);
			styleKeyLookupTable = new Hashtable(14);
			styleNameLookupArray = new string[14];
			RegisterStyle("background-color", HtmlTextWriterStyle.BackgroundColor);
			RegisterStyle("background-image", HtmlTextWriterStyle.BackgroundImage);
			RegisterStyle("border-collapse", HtmlTextWriterStyle.BorderCollapse);
			RegisterStyle("border-color", HtmlTextWriterStyle.BorderColor);
			RegisterStyle("border-style", HtmlTextWriterStyle.BorderStyle);
			RegisterStyle("border-width", HtmlTextWriterStyle.BorderWidth);
			RegisterStyle("color", HtmlTextWriterStyle.Color);
			RegisterStyle("font-family", HtmlTextWriterStyle.FontFamily);
			RegisterStyle("font-size", HtmlTextWriterStyle.FontSize);
			RegisterStyle("font-style", HtmlTextWriterStyle.FontStyle);
			RegisterStyle("font-weight", HtmlTextWriterStyle.FontWeight);
			RegisterStyle("height", HtmlTextWriterStyle.Height);
			RegisterStyle("text-decoration", HtmlTextWriterStyle.TextDecoration);
			RegisterStyle("width", HtmlTextWriterStyle.Width);
		}

		private HtmlHelper()
		{
		}

		#endregion

		#region private structs

		private struct AttributeInformation
		{
			public bool encode;
			public string name;
 
			public AttributeInformation(string name, bool encode)
			{
				this.name = name;
				this.encode = encode;
			}
		}

		private struct RenderAttribute
		{
			public bool encode;
			public HtmlTextWriterAttribute key;
			public string name;
			public string value;
		}

		private struct RenderStyle
		{
			public HtmlTextWriterStyle key;
			public string name;
			public string value;
		}

		private struct TagInformation
		{
			public string closingTag;
			public string name;
			public TagType tagType;

			public TagInformation(string name, TagType tagType, string closingTag)
			{
				this.name = name;
				this.tagType = tagType;
				this.closingTag = closingTag;
			}
		}

		private enum TagType
		{
			Inline = 0,
			NonClosing = 1,
			Other = 2
		}

		#endregion

		#region Style Methods

		private static void RegisterAttribute(string name, HtmlTextWriterAttribute key)
		{
			RegisterAttribute(name, key, false);
		}

		private static void RegisterAttribute(string name, HtmlTextWriterAttribute key, bool fEncode)
		{
			string lookup = name.ToLower(CultureInfo.InvariantCulture);
			attrKeyLookupTable.Add(lookup, key);
			if ((int)key < attrNameLookupArray.Length)
			{
				attrNameLookupArray[(int)key] = new AttributeInformation(name, fEncode);
			}
		}

		private static void RegisterStyle(string name, HtmlTextWriterStyle key)
		{
			string lookup = name.ToLower(CultureInfo.InvariantCulture);
			styleKeyLookupTable.Add(lookup, key);
			if ((int)key < styleNameLookupArray.Length)
			{
				styleNameLookupArray[(int)key] = name;
			}
		}

		private static void RegisterTag(string name, HtmlTextWriterTag key)
		{
			RegisterTag(name, key, TagType.Other);
		}

		private static void RegisterTag(string name, HtmlTextWriterTag key, TagType type)
		{
			string lookup = name.ToLower(CultureInfo.InvariantCulture);
			tagKeyLookupTable.Add(lookup, key);
			string tag = null;
			if ((type != TagType.NonClosing) && (key != HtmlTextWriterTag.Unknown))
			{
				tag = string.Concat("</", lookup, ">");
			}
			if ((int)key < tagNameLookupArray.Length)
			{
				tagNameLookupArray[(int)key] = new TagInformation(name, type, tag);
			}
		}

		/// <summary>
		/// Retrieves the <c ref="HtmlTextWriterAttribute">HtmlTextWriterAttribute</c>
		/// for the given html string attribute expression
		/// </summary>
		/// <param name="attrName"></param>
		/// <returns></returns>
		public static HtmlTextWriterAttribute GetAttributeKey(string attrName)
		{
			if ((attrName == null) || (attrName.Length <= 0))
			{
				return ((HtmlTextWriterAttribute)(-1));
			}
			object obj1 = attrKeyLookupTable[attrName.ToLower(CultureInfo.InvariantCulture)];
			if (obj1 != null)
			{
				return ((HtmlTextWriterAttribute) obj1);
			}
			return HtmlTextWriterAttribute.Rules;
		}

		/// <summary>
		/// Retrieves the string to be used in html of the given attribute
		/// </summary>
		/// <param name="attrKey"></param>
		/// <returns></returns>
		public static string GetAttributeName(HtmlTextWriterAttribute attrKey)
		{
			if ((attrKey >= HtmlTextWriterAttribute.Accesskey) && ((int)attrKey < attrNameLookupArray.Length))
			{
				return attrNameLookupArray[(int)attrKey].name;
			}
			return string.Empty;
		}

		/// <summary>
		/// Retrieves the <c ref="HtmlTextWriterStyle">HtmlTextWriterStyle</c> for the
		/// given html string style attribute
		/// </summary>
		/// <param name="styleName"></param>
		/// <returns></returns>
		public static HtmlTextWriterStyle GetStyleKey(string styleName)
		{
			if ((styleName == null) || (styleName.Length <= 0))
			{
				return ((HtmlTextWriterStyle)(-1));
			}
			object obj1 = styleKeyLookupTable[styleName.ToLower(CultureInfo.InvariantCulture)];
			if (obj1 != null)
			{
				return ((HtmlTextWriterStyle) obj1);
			}
			return HtmlTextWriterStyle.TextDecoration;
		}

		public static string GetStyleName(HtmlTextWriterStyle styleKey)
		{
			if ((styleKey >= HtmlTextWriterStyle.BackgroundColor) && ((int)styleKey < styleNameLookupArray.Length))
			{
				return styleNameLookupArray[(int)styleKey];
			}
			return string.Empty;
		}

		public static HtmlTextWriterTag GetTagKey(string tagName)
		{
			if ((tagName == null) || (tagName.Length <= 0))
			{
				return HtmlTextWriterTag.Unknown;
			}
			object obj1 = tagKeyLookupTable[tagName.ToLower(CultureInfo.InvariantCulture)];
			if (obj1 != null)
			{
				return ((HtmlTextWriterTag) obj1);
			}
			return HtmlTextWriterTag.S;
		}

		public static string GetTagName(HtmlTextWriterTag tagKey)
		{
			int num1 = ((int) tagKey);
			if ((num1 >= 0) && (num1 < tagNameLookupArray.Length))
			{
				return tagNameLookupArray[num1].name;
			}
			return string.Empty;
		}
 
		/// <summary>
		/// Converts a Style object to a string which can be used for the 'style' attribute
		/// of an Html item.
		/// </summary>
		/// <param name="style"></param>
		/// <returns></returns>
		public static string GetStyleString(Style style)
		{
			StringWriter sw = new StringWriter();
			HtmlTextWriter styleWriter = new HtmlTextWriter(sw);

			if(style.ForeColor != Color.Empty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.Color), ColorTranslator.ToHtml(style.ForeColor));
			}
			if(style.BackColor != Color.Empty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.BackgroundColor), ColorTranslator.ToHtml(style.BackColor));
			}
			if(style.BorderColor != Color.Empty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.BorderColor), ColorTranslator.ToHtml(style.BorderColor));
			}
			if(!style.BorderWidth.IsEmpty)
			{
				if(style.BorderStyle == BorderStyle.NotSet)
				{
					if(style.BorderWidth.Value != 0f)
					{
						styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.BorderStyle), "solid");
					}
				}
				else
				{
					styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.BorderStyle), Enum.Format(typeof(BorderStyle), style.BorderStyle, "G"));
				}
			}
			else if(style.BorderStyle != BorderStyle.NotSet)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.BorderStyle), Enum.Format(typeof(BorderStyle), style.BorderStyle, "G"));
			}

			FontInfo font = style.Font;
			string[] fontNames = font.Names;
			if(fontNames.Length > 0)
			{
                DelimitedList<string> names = new DelimitedList<string>(fontNames);
                styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.FontFamily), names.ToString());
			}

			FontUnit size = font.Size;
			if(!size.IsEmpty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.FontSize), size.ToString(CultureInfo.InvariantCulture));
			}
			if(font.Bold)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.FontWeight), "bold");
			}
			if(font.Italic)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.FontStyle), "italic");
			}
			string textDecoration = String.Empty;
			if(font.Underline)
			{
				textDecoration = "underline";
			}
			if(font.Overline)
			{
				string.Concat(textDecoration, " overline");
			}
			if(font.Strikeout)
			{
				string.Concat(textDecoration, " line-through");
			}
			if(textDecoration.Length > 0)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.TextDecoration), textDecoration);
			}
			if(!style.Height.IsEmpty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.Height), style.Height.ToString(CultureInfo.InvariantCulture));
			}
			if(!style.Width.IsEmpty)
			{
				styleWriter.WriteStyleAttribute(GetStyleName(HtmlTextWriterStyle.Width), style.Width.ToString(CultureInfo.InvariantCulture));
			}

			return sw.ToString();
		}

		public static void WriteStyle(Style style, HtmlTextWriter writer)
		{	
			if(style.CssClass.Length > 0)
				writer.WriteAttribute(GetAttributeName(HtmlTextWriterAttribute.Class), style.CssClass);

			string styleString = GetStyleString(style);
			if(styleString.Length > 0)
				writer.WriteAttribute(GetAttributeName(HtmlTextWriterAttribute.Style), GetStyleString(style));
		}

		#endregion
    }
}