#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.IO;
using System.Drawing;
using System.ComponentModel;
using System.Web;

#endregion

namespace Web
{
    public class StyleBlock : Panel
    {
        #region Properties

        public string BlockTag
        {
            get
            {
                object o = ViewState["BlockTag"];
                return o == null ? "style" : (string)o;
            }
            set
            {
                ViewState["BlockTag"] = value;
            }
        }

        public string Type
        {
            get
            {
                object o = ViewState["Type"];
                return o == null ? "text/css" : (string)o;
            }
            set
            {
                ViewState["Type"] = value;
            }
        }

        public string IncludePath
        {
            get
            {
                object o = ViewState["IncludePath"];
                return o == null ? String.Empty : (string)o;
            }
            set
            {
                ViewState["IncludePath"] = value;
            }
        }

        public string ResourceName
        {
            get
            {
                object o = ViewState["ResourceName"];
                return o == null ? String.Empty : (string)o;
            }
            set
            {
                ViewState["ResourceName"] = value;
            }
        }

        public string ResourceAssembly
        {
            get
            {
                object o = ViewState["ResourceAssembly"];
                return o == null ? String.Empty : (string)o;
            }
            set
            {
                ViewState["ResourceAssembly"] = value;
            }
        }

        public string Contents
        {
            get
            {
                object o = ViewState["Contents"];
                return o == null ? String.Empty : (string)o;
            }
            set
            {
                ViewState["Contents"] = value;
            }
        }

        public CssMediaType Media
        {
            get
            {
                object o = ViewState["Media"];
                return o == null ? CssMediaType.Screen : (CssMediaType)o;
            }
            set
            {
                ViewState["Media"] = value;
            }
        }

        #endregion

        #region Overidden Methods

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            if (Visible)
            {
                if (!String.IsNullOrEmpty(IncludePath))
                {
                    Uri uri = new Uri(IncludePath, UriKind.RelativeOrAbsolute);
                    if (uri.IsAbsoluteUri)
                    {
                        Page.Header.Controls.Add(new LiteralControl(String.Format("\n<link href=\"{0}\" rel=\"stylesheet\" type=\"text/css\" media=\"{1}\" />\n", IncludePath, Media)));
                    }
                    else
                    {
                        Page.Header.Controls.Add(new LiteralControl(String.Format("\n<link href=\"{0}\" rel=\"stylesheet\" type=\"text/css\" media=\"{1}\" />\n", VirtualPathUtility.ToAbsolute(IncludePath), Media)));
                    }
                }
                else
                {
                    RenderBlockContents();
                }
            }
        }

        protected void RenderBlockContents()
        {
            string renderedContents = String.Empty;
            if (!String.IsNullOrEmpty(ResourceName) && !String.IsNullOrEmpty(ResourceAssembly))
            {
                renderedContents = ResourceHelper.ResourceToString(ResourceName, ResourceAssembly);
            }
            else
            {
                StringBuilder sb = new StringBuilder();
                StringWriter sw = new StringWriter(sb);
                HtmlTextWriter tw = new HtmlTextWriter(sw);
                base.RenderContents(tw);
                if (!String.IsNullOrEmpty(Contents))
                    sb.Append(Contents);
                renderedContents = sb.ToString();
                sw.Close();
                tw.Close();
            }
            Page.Header.Controls.Add(new LiteralControl(String.Format("\n<{0} type=\"{1}\" media=\"{3}\">\n{2}\n</{0}>\n", BlockTag, Type, renderedContents, Media)));
        }

        protected override void Render(HtmlTextWriter writer)
        {
        }

        #endregion
    }

    public enum CssMediaType
    {
        All,
        Aural,
        Braille,
        Embossed,
        Handheld,
        Print,
        Projection,
        Screen,
        Tv
    }
}
